﻿# script d'installation des applications via winget
# executé avant wpkg, il est donc prioritaire sur celui-ci. Idéalement wpkg n'a plus rien à faire ensuite.
#
# on envoie au se4fs un json contenant la liste des applications. Celle-ci est analysée et
#  - on récupère un json contenant la listes des applications à désinstaller
#  - on récupère la liste des applications installer mettre à jour.
# on envoie à nouveau le résultat pour la mise à jour des rapports
#
# si une application définit explicitement une version dans wpkg, on ne la met pas à jour.
# dans le cas contraire on met à jour sysématiquement.


# utilisation des commandlets
Try {
#   Get-WinGetPackage
    Assert-WinGetPackageManager
} Catch {
    Install-Module -Name Microsoft.WinGet.Client -Repository PSGallery -Scope AllUsers -Force
}
#Install-PackageProvider NuGet -Force
#Set-PSRepository PSGallery -InstallationPolicy Trusted
#Install-Module Microsoft.Winget.Client -Confirm:$False -Force -AllowClobber

Import-Module -Name Microsoft.Winget.Client
Get-Date -f dd-MM-yyyy | Out-File "C:\netinst\winget.log" -Force
# récupération de la liste de tous les logiciels installés avec winget list
# permet d'avoir la liste des installations hors winget pour remontée, et pour desinstallation
# ces infos ne sont pas utilisées pour la mise à jour et l'installation via winget.
Try {
    $list = Get-WinGetPackage | ConvertTo-Json
    #Write-Output $list
} Catch {
    Write-Host "Failed to list applications with Winget..." -ForegroundColor Red | Out-File "C:\netinst\winget.log" -Append
    Exit 1
}

$Uri = "http://${env:SE4FS}/wpkg/winget_out.php"
$Form = @{
    machine  = $env:ComputerName
    list     = $list
    action   = 'list'
}

Try {
    $winget = Invoke-RestMethod -Uri $Uri -Method Post -Body $Form
} Catch {
    Write-Host "Failed to upload installed apps... Maybe you need to add winget = `"1`" in /etc/sambaedu/sambaedu.conf.d/clients.conf" -ForegroundColor Red
    Exit 1
}

# uninstall

$uninstall = $winget.uninstall

ForEach ( $app in $uninstall ) {
    Write-Host "To be uninstalled :" $app.Id -ForegroundColor Yellow
    Uninstall-WingetPackage -Id $app.Id | Out-File "C:\netinst\winget.log" -Append
}

# install
 $install = $winget.install

ForEach ( $app in $install ) {
    $Args = @{
        "Id" = $app.Id
        "Scope" = "System"
        "Source" = $app.Source
    }
    if ($app.Custom) {
        $Args.Add("Custom", $app.Custom)
    }
    if ($app.Override) {
        $Args.Add("Override", $app.Override)
    }

    Write-Host "To be installed :" $app.Id " from source : " $app.Source -ForegroundColor Green
    Install-WingetPackage @Args | Out-File "C:\netinst\winget.log" -Append
}

# upgrade
$upgrade = $winget.upgrade

ForEach ( $app in $upgrade ) {
    $Args = @{
        "Id" = $app.Id
        "Version" = $app.AvailableVersion
        "Scope" = "System"
        "Source" = $app.Source
    }
    if ($app.Custom) {
        $Args.Add("Custom", $app.Custom)
    }
    if ($app.Override) {
        $Args.Add("Override", $app.Override)
    }

    Write-Host "To be upgraded :" $app.Id  $app.Version ">= " $app.AvailableVersion -ForegroundColor Blue
    Update-WingetPackage @Args | Out-File "C:\netinst\winget.log" -Append
}
