﻿# script d'installation de winget
# installe et met à jour winget, et install powershell 7 qui est obligatiore pour pouvoir utiliser
# les commandlets winget en mode System...
# Add-AppxPackage -RegisterByFamilyName -MainPackage Microsoft.DesktopAppInstaller_8wekyb3d8bbwe

Function Set-PathVariable {
    param (
        [string]$AddPath,
        [string]$RemovePath,
        [ValidateSet('Process', 'User', 'Machine')]
        [string]$Scope = 'Process'
    )
    $regexPaths = @()
    if ($PSBoundParameters.Keys -contains 'AddPath') {
        $regexPaths += [regex]::Escape($AddPath)
    }

    if ($PSBoundParameters.Keys -contains 'RemovePath') {
        $regexPaths += [regex]::Escape($RemovePath)
    }
    
    $arrPath = [System.Environment]::GetEnvironmentVariable('PATH', $Scope) -split ';'
    foreach ($path in $regexPaths) {
        $arrPath = $arrPath | Where-Object { $_ -notMatch "^$path\\?" }
    }
    $value = ($arrPath + $addPath) -join ';'
    [System.Environment]::SetEnvironmentVariable('PATH', $value, $Scope)
}

#Check Winget Install
Write-Host "Checking if Winget is installed" -ForegroundColor Yellow
$TestWinget = Get-AppxProvisionedPackage -Online | Where-Object {$_.DisplayName -eq "Microsoft.DesktopAppInstaller"}
If ([Version]$TestWinGet. Version -ge "2024.423.111.0") {
    Write-Host "WinGet is Installed" -ForegroundColor Green
} Else {
    #WebClient
    $dc = New-Object net.webclient
    $dc.UseDefaultCredentials = $true
    $dc.Headers.Add("user-agent", "Internet Explorer")
    $dc.Headers.Add("X-FORMS_BASED_AUTH_ACCEPTED", "f")

    #temp folder
    $InstallerFolder = $(Join-Path $env:ProgramData CustomScripts)
    If (!(Test-Path $InstallerFolder)) {
        New-Item -Path $InstallerFolder -ItemType Directory -Force -Confirm:$false
    }

    #Download WinGet MSIXBundle
    Write-Host "Not installed. Downloading WinGet..."
    $WinGetURL = "https://aka.ms/getwinget"
    $dc.DownloadFile($WinGetURL, "$InstallerFolder\Microsoft.DesktopAppInstaller_8wekyb3d8bbwe.msixbundle")

    If (Test-Path "$InstallerFolder\Microsoft.DesktopAppInstaller_8wekyb3d8bbwe.msixbundle" -PathType Leaf) {
        #Install WinGet MSIXBundle
        Try {
            Write-Host "Installing MSIXBundle for App Installer..."
            Add-AppxProvisionedPackage -Online -PackagePath "$InstallerFolder\Microsoft.DesktopAppInstaller_8wekyb3d8bbwe.msixbundle" -SkipLicense
            Add-AppPackage -ForceApplicationShutdown  -PackagePath "$InstallerFolder\Microsoft.DesktopAppInstaller_8wekyb3d8bbwe.msixbundle" -SkipLicense
            Write-Host "Installed MSIXBundle for App Installer" -ForegroundColor Green
        } Catch {
            Write-Host "Failed to install MSIXBundle for App Installer..." -ForegroundColor Red
            Exit 1
        }
    } Else {
        Write-Host "Failed to download or save MSIXBundle for App Installer..." -ForegroundColor Red
        Exit 1
    }
}
$ResolveWingetPath = Resolve-Path "C:\Program Files\WindowsApps\Microsoft.DesktopAppInstaller_*_x64__8wekyb3d8bbwe"
If ($ResolveWingetPath) {
    $WingetPath = $ResolveWingetPath[-1].Path
#    Write-Host "WinGet path is :`n$WingetPath" -ForegroundColor Green
} Else {
    Write-Host "Unable to find WinGet path..." -ForegroundColor Red
    Exit 1
}
Set-PathVariable --AddPath $WingetPath --Scope Machine

$config
Set-Location $WingetPath
$count = 0
While ($count -lt 144) {
    Try {
        .\winget.exe install --Id Microsoft.Powershell --force --disable-interactivity --Silent --Scope machine --accept-source-agreements --accept-package-agreements
        $count = 144
    } Catch {
        Write-Host "Unable to install Powershell 7, will retry every 10 minutes during 24h..." -ForegroundColor Red
        $count++
        Start-sleep -Seconds 600
    }
}

